@@ -1,4 +1,3 @@
-# vim:fileencoding=utf-8 
 #
 # greetingInGreek.py
 #
@@ -6,15 +5,14 @@
 #
 # Copyright 2004-2016, by Paul McGuire
 #
-from pyparsing import Word, pyparsing_unicode
+from pyparsing import Word, pyparsing_unicode as ppu
 
 # define grammar
-alphas = pyparsing_unicode.Greek.alphas
-greet = Word(alphas) + ',' + Word(alphas) + '!' 
+alphas = ppu.Greek.alphas
+greet = Word(alphas) + "," + Word(alphas) + "!"
 
 # input string
 hello = "Καλημέρα, κόσμε!"
 
 # parse input string
 print(greet.parseString(hello))
-
